package eu.eudat.data.dao.entities;

import eu.eudat.data.dao.DatabaseAccess;
import eu.eudat.data.dao.criteria.ProjectCriteria;
import eu.eudat.data.dao.databaselayer.service.DatabaseService;
import eu.eudat.data.entities.Project;
import eu.eudat.data.entities.UserInfo;
import eu.eudat.queryable.QueryableList;
import eu.eudat.types.project.ProjectStateType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import javax.persistence.criteria.JoinType;
import java.util.Date;
import java.util.UUID;
import java.util.concurrent.CompletableFuture;

@Component("projectDao")
public class ProjectDaoImpl extends DatabaseAccess<Project> implements ProjectDao {

    @Autowired
    public ProjectDaoImpl(DatabaseService<Project> databaseService) {
        super(databaseService);
    }

    @Override
    public QueryableList<Project> getWithCriteria(ProjectCriteria criteria) {
        QueryableList<Project> query = getDatabaseService().getQueryable(Project.class);
        if (criteria.getLike() != null && !criteria.getLike().isEmpty())
            query.where((builder, root) ->
                    builder.or(builder.like(builder.upper(root.get("label")), "%" + criteria.getLike().toUpperCase() + "%"),
                    builder.or(builder.like(builder.upper(root.get("description")), "%" + criteria.getLike().toUpperCase() + "%"))));
        if (criteria.getPeriodEnd() != null)
            query.where((builder, root) -> builder.lessThan(root.get("enddate"), criteria.getPeriodEnd()));
        if (criteria.getPeriodStart() != null)
            query.where((builder, root) -> builder.greaterThan(root.get("startdate"), criteria.getPeriodStart()));
        if (criteria.getReference() != null)
            query.where((builder, root) -> builder.equal(root.get("reference"), criteria.getReference()));
        if (criteria.getProjectStateType() != null) {
            if (criteria.getProjectStateType().equals(ProjectStateType.FINISHED.getValue()))
                query.where((builder, root) -> builder.lessThan(root.get("enddate"), new Date()));
            if (criteria.getProjectStateType().equals(ProjectStateType.ONGOING.getValue()))
                query.where((builder, root) ->
                        builder.or(builder.greaterThan(root.get("enddate"), new Date())
                                , builder.isNull(root.get("enddate"))));
        }
        query.where((builder, root) -> builder.notEqual(root.get("status"), Project.Status.DELETED.getValue()));
        return query;
    }

    @Override
    public Project createOrUpdate(Project item) {
        return getDatabaseService().createOrUpdate(item, Project.class);
    }

    @Override
    public Project find(UUID id) {
        return getDatabaseService().getQueryable(Project.class).where((builder, root) -> builder.equal((root.get("id")), id)).getSingle();
    }

    @Override
    public void delete(Project item) {
        this.getDatabaseService().delete(item);
    }

    @Override
    public QueryableList<Project> asQueryable() {
        return this.getDatabaseService().getQueryable(Project.class);
    }

    public QueryableList<Project> getAuthenticated(QueryableList<Project> query, UserInfo principal) {
        query.where((builder, root) -> builder.or(builder.equal(root.get("creationUser"), principal), builder.isMember(principal, root.join("dmps", JoinType.LEFT).get("users")))).distinct();
        return query;
    }

    @Async
    @Override
    public CompletableFuture<Project> createOrUpdateAsync(Project item) {
        return CompletableFuture.supplyAsync(() -> this.createOrUpdate(item));
    }

    @Override
    public Project find(UUID id, String hint) {
        throw new UnsupportedOperationException();
    }
}
