import { DatasetProfileDefinitionModel } from '../DatasetProfileDefinitionModel';
import { DatasetProfileModel } from '../datasetprofile/DatasetProfileModel';
import { DataManagementPlanModel } from '../data-managemnt-plans/DataManagementPlanModel';
import { Serializable } from '../Serializable';
import { ValidationContext } from '../../utilities/validators/ValidationContext';
import { FormGroup, FormBuilder, FormControl, Validators } from '@angular/forms';
import { BackendErrorValidator } from '../../utilities/validators/BackendErrorValidator';
import { BaseErrorModel } from '../error/BaseErrorModel';
import { ExternalSourcesItemModel } from '../external-sources/ExternalSourcesItemModel';
import { ServiceModel } from '../services/ServiceModel';
import { JsonSerializer } from '../../utilities/JsonSerializer';
import { RegisterModel } from '../registers/RegisterModel';
import { DataRepositoryModel } from '../dataRepositories/DataRepositoryModel';
import { ExternalDatasetModel } from '../../models/external-dataset/ExternalDatasetModel';
import { TagModel } from '../tags/TagModel';

export enum DatasetStatus {
	Draft = 0,
	Finalised = 1,
	Deleted = 99
}

export class DatasetWizardModel implements Serializable<DatasetWizardModel> {
	public id: string;
	public label: string;
	public profile: DatasetProfileModel = new DatasetProfileModel();
	public uri: String;
	public status: number;
	public description: String;
	public services: ServiceModel[] = [];
	public registries: RegisterModel[] = [];
	public dataRepositories: DataRepositoryModel[] = [];
	public tags: TagModel[] = [];
	public externalDatasets: ExternalDatasetModel[] = [];
	public dmp: DataManagementPlanModel = new DataManagementPlanModel();
	public datasetProfileDefinition;
	public errorModel: BaseErrorModel = new BaseErrorModel();

	fromJSONObject(item: any): DatasetWizardModel {
		this.id = item.id;
		this.label = item.label;
		this.profile = item.profile;
		this.uri = item.uri;
		this.status = item.status;
		this.description = item.description;
		this.services = JsonSerializer.fromJSONArray(item.services, ServiceModel);
		this.registries = JsonSerializer.fromJSONArray(item.registries, RegisterModel);
		this.dataRepositories = JsonSerializer.fromJSONArray(item.dataRepositories, DataRepositoryModel);
		this.dmp = JsonSerializer.fromJSONObject(item.dmp, DataManagementPlanModel);
		this.profile = JsonSerializer.fromJSONObject(item.profile, DatasetProfileModel);
		this.externalDatasets = JsonSerializer.fromJSONArray(item.externalDatasets, ExternalDatasetModel);
		this.datasetProfileDefinition = JsonSerializer.fromJSONObject(item.datasetProfileDefinition, DatasetProfileDefinitionModel);
		this.tags = JsonSerializer.fromJSONArray(item.tags, TagModel);
		return this;
	}

	buildForm(context: ValidationContext = null, disabled: boolean = false): FormGroup {
		if (context == null) { context = this.createValidationContext(); }
		const formBuilder = new FormBuilder();
		const formGroup = formBuilder.group({
			id: [{ value: this.id, disabled: disabled }, context.getValidation('id').validators],
			label: [{ value: this.label, disabled: disabled }, context.getValidation('label').validators],
			uri: [{ value: this.uri, disabled: disabled }, context.getValidation('uri').validators],
			status: [{ value: this.status, disabled: disabled }, context.getValidation('status').validators],
			description: [{ value: this.description, disabled: disabled }, context.getValidation('description').validators],
			dmp: [{ value: this.dmp, disabled: disabled }, context.getValidation('dmp').validators],
		});

		const externalDatasetsFormArray = new Array<FormGroup>();
		if (this.externalDatasets && this.externalDatasets.length > 0) {
			this.externalDatasets.forEach(item => {
				externalDatasetsFormArray.push(item.buildForm(context.getValidation('externalDatasets').descendantValidations, disabled));
			});
		} else {
			//externalDatasetsFormArray.push(new ExternalDatasetModel().buildForm(context.getValidation('externalDatasets').descendantValidations, disabled));
		}
		formGroup.addControl('externalDatasets', formBuilder.array(externalDatasetsFormArray));

		const tagsFormArray = new Array<FormGroup>();
		if (this.tags && this.tags.length > 0) {
			this.tags.forEach(item => {
				tagsFormArray.push(item.buildForm(context.getValidation('tags').descendantValidations, disabled));
			});
		} else {
			//externalDatasetsFormArray.push(new ExternalDatasetModel().buildForm(context.getValidation('externalDatasets').descendantValidations, disabled));
		}
		formGroup.addControl('tags', formBuilder.array(tagsFormArray));

		const registriesFormArray = new Array<FormGroup>();
		if (this.registries && this.registries.length > 0) {
			this.registries.forEach(item => {
				registriesFormArray.push(item.buildForm(context.getValidation('registries').descendantValidations, disabled));
			});
		} else {
			//externalDatasetsFormArray.push(new ExternalDatasetModel().buildForm(context.getValidation('externalDatasets').descendantValidations, disabled));
		}
		formGroup.addControl('registries', formBuilder.array(registriesFormArray));

		const dataRepositoriesFormArray = new Array<FormGroup>();
		if (this.dataRepositories && this.dataRepositories.length > 0) {
			this.dataRepositories.forEach(item => {
				dataRepositoriesFormArray.push(item.buildForm(context.getValidation('dataRepositories').descendantValidations, disabled));
			});
		} else {
			//externalDatasetsFormArray.push(new ExternalDatasetModel().buildForm(context.getValidation('externalDatasets').descendantValidations, disabled));
		}
		formGroup.addControl('dataRepositories', formBuilder.array(dataRepositoriesFormArray));

		const servicesFormArray = new Array<FormGroup>();
		if (this.services && this.services.length > 0) {
			this.services.forEach(item => {
				servicesFormArray.push(item.buildForm(context.getValidation('services').descendantValidations, disabled));
			});
		} else {
			//externalDatasetsFormArray.push(new ExternalDatasetModel().buildForm(context.getValidation('externalDatasets').descendantValidations, disabled));
		}
		formGroup.addControl('services', formBuilder.array(servicesFormArray));

		if (this.datasetProfileDefinition) { formGroup.addControl('datasetProfileDefinition', this.datasetProfileDefinition.buildForm()); }
		formGroup.addControl('profile', this.profile.buildForm());
		return formGroup;
	}

	createValidationContext(): ValidationContext {
		const baseContext: ValidationContext = new ValidationContext();
		baseContext.validation.push({ key: 'id', validators: [BackendErrorValidator(this.errorModel, 'id')] });
		baseContext.validation.push({ key: 'label', validators: [Validators.required, BackendErrorValidator(this.errorModel, 'label')] });
		baseContext.validation.push({ key: 'profile', validators: [Validators.required, BackendErrorValidator(this.errorModel, 'profile')] });
		baseContext.validation.push({ key: 'uri', validators: [BackendErrorValidator(this.errorModel, 'uri')] });
		baseContext.validation.push({ key: 'status', validators: [BackendErrorValidator(this.errorModel, 'status')] });
		baseContext.validation.push({ key: 'description', validators: [BackendErrorValidator(this.errorModel, 'description')] });
		baseContext.validation.push({ key: 'services', validators: [BackendErrorValidator(this.errorModel, 'services')] });
		baseContext.validation.push({ key: 'registries', validators: [BackendErrorValidator(this.errorModel, 'registries')] });
		baseContext.validation.push({ key: 'dataRepositories', validators: [BackendErrorValidator(this.errorModel, 'dataRepositories')] });
		baseContext.validation.push({ key: 'externalDatasets', validators: [BackendErrorValidator(this.errorModel, 'externalDatasets')] });
		baseContext.validation.push({ key: 'dmp', validators: [BackendErrorValidator(this.errorModel, 'dmp')] });   //TODO
		baseContext.validation.push({ key: 'datasetProfileDefinition', validators: [BackendErrorValidator(this.errorModel, 'datasetProfileDefinition')] });   //TODO
		baseContext.validation.push({ key: 'tags', validators: [BackendErrorValidator(this.errorModel, 'datasetProfileDefinition')] });   //TODO

		return baseContext;
	}
}
