import { Component, OnDestroy, OnInit } from '@angular/core';
import { FormBuilder, FormControl, FormGroup } from '@angular/forms';
import { ActivatedRoute, Params, Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import * as moment from 'moment-timezone';
import { takeUntil } from 'rxjs/operators';
import { Observable } from 'rxjs/Rx';
import { BaseComponent } from '../../core/common/base/base.component';
import { DataManagementPlanModel } from '../../models/data-managemnt-plans/DataManagementPlanModel';
import { UserListingModel } from '../../models/users/UserListingModel';
import { AuthService } from '../../services/auth/auth.service';
import { UserReferenceService } from '../../services/user-reference/user-reference-data.service';
import { CultureService } from '../../utilities/culture/culture-service';
import { CultureInfo } from '../../utilities/culture/models/culture-info';

const availableLanguages: any[] = require('../../../assets/resources/language.json');

@Component({
	selector: 'app-user-profile',
	templateUrl: './user-profile.component.html',
	styleUrls: ['./user-profile.component.scss'],
	providers: [
		UserReferenceService,
		CultureService
	]
})
export class UserProfileComponent extends BaseComponent implements OnInit, OnDestroy {

	user: Observable<UserListingModel>;
	currentUserId: string;
	cultures: Observable<CultureInfo[]>;
	timezones: Observable<any[]>;
	editMode = false;
	languages = availableLanguages;

	formGroup: FormGroup;
	constructor(
		private userReferenceService: UserReferenceService,
		private route: ActivatedRoute,
		private router: Router,
		private authService: AuthService,
		private language: TranslateService,
		private cultureService: CultureService,
		private translate: TranslateService,
	) { super(); }

	ngOnInit() {
		this.route.params
			.pipe(takeUntil(this._destroyed))
			.subscribe((params: Params) => {
				this.currentUserId = params['id'];
				const userId = params['id'] === this.authService.current().id ? 'me' : params['id'];
				this.user = this.userReferenceService.getUser(userId).map(result => {
					result['additionalinfo'] = JSON.parse(result['additionalinfo']);
					this.formGroup = new FormBuilder().group({
						language: new FormControl(result['additionalinfo']['language'] ? availableLanguages.filter(x => x.value === result['additionalinfo']['language']['value']).pop() : ''),
						timezone: new FormControl(result['additionalinfo']['timezone']),
						culture: new FormControl(result['additionalinfo']['culture'])
					});
					//this.formGroup.get('language').valueChanges.pipe(takeUntil(this._destroyed)).subscribe(x => { if (x) this.translate.use(x.value) })
					this.formGroup.get('timezone').valueChanges
						.pipe(takeUntil(this._destroyed))
						.subscribe(x => { if (x) { this.timezones = this._filterTimezone(x); } });
					this.formGroup.get('culture').valueChanges
						.pipe(takeUntil(this._destroyed))
						.subscribe(x => { if (x) { this.cultures = this._filterCulture(x); } });
					this.formGroup.disable();
					return result;
				});
			});

	}

	ngOnDestroy(): void {
	}

	getUserRole(dmp: DataManagementPlanModel) {
		if (dmp.creator.id === this.currentUserId) { return this.language.instant('USER-PROFILE.DMPS.CREATOR'); } else if (dmp.associatedUsers.map(x => x.id).indexOf(this.currentUserId) !== -1) { return this.language.instant('USER-PROFILE.DMPS.MEMBER'); }
		return '';
	}

	showAllDmps() {
		this.router.navigate(['/dmps']);
	}

	navigateToDmp(dmp: DataManagementPlanModel) {
		this.router.navigate(['/dmps/edit/' + dmp.id]);
	}

	private _filterTimezone(value: string): Observable<any[]> {
		if (value && typeof value === 'string') {
			const filterValue = value.toLowerCase();
			return Observable.of(moment.tz.names().filter(option => option.toLowerCase().includes(filterValue)));
		} else {
			return Observable.of(moment.tz.names());
		}
	}

	private _filterCulture(value: string): Observable<any[]> {
		if (value && typeof value === 'string') {
			const filterValue = value.toLowerCase();
			return Observable.of(this.cultureService.getCultureValues().filter(option => option.displayName.toLowerCase().includes(filterValue)));
		} else {
			return Observable.of(this.cultureService.getCultureValues());
		}
	}

	displayFn(culture?: CultureInfo): string | undefined {
		return culture ? culture.displayName + '-' + culture.nativeName : undefined;
	}

	save() {

	}

	public unlock() {
		this.editMode = true;
		this.formGroup.enable();
	}

	public lock() {
		this.userReferenceService.updateUserSettings(this.formGroup.value)
			.pipe(takeUntil(this._destroyed))
			.subscribe(
				x => {
					this.editMode = false;
					this.translate.use(this.formGroup.value.language);
					this.authService.current().culture = this.formGroup.value.culture.name;
					this.formGroup.disable();
					this.authService.me()
						.pipe(takeUntil(this._destroyed))
						.subscribe(result => window.location.reload());
				},
				error => {
					console.log(error);
				});
	}

}
